@@ -1,25 +1,41 @@
 /*
- * Asterisk -- A telephony toolkit for Linux.
+ * Asterisk -- An open source telephony toolkit.
  *
- * Verbose application
- * 
- * Copyright (c) 2004 Tilghman Lesher.  All rights reserved.
+ * Copyright (c) 2004 - 2005 Tilghman Lesher.  All rights reserved.
  *
  * Tilghman Lesher <app_verbose_v001@the-tilghman.com>
  *
  * This code is released by the author with no restrictions on usage.
  *
+ * See http://www.asterisk.org for more information about
+ * the Asterisk project. Please do not directly contact
+ * any of the maintainers of this project for assistance;
+ * the project provides a web site, mailing lists and IRC
+ * channels for your use.
+ *
+ */
+
+/*! \file
+ *
+ * \brief Verbose logging application
+ *
+ * \ingroup applications
  */
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <string.h>
-#include <asterisk/options.h>
-#include <asterisk/logger.h>
-#include <asterisk/channel.h>
-#include <asterisk/pbx.h>
-#include <asterisk/module.h>
+
+#include "asterisk.h"
+
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 7221 $")
+
+#include "asterisk/options.h"
+#include "asterisk/logger.h"
+#include "asterisk/channel.h"
+#include "asterisk/pbx.h"
+#include "asterisk/module.h"
 
 
 static char *tdesc = "Send verbose output";
@@ -30,8 +46,7 @@ static char *verbose_synopsis = "Send arbitrary text to verbose output";
 
 static char *verbose_descrip =
 "Verbose([<level>|]<message>)\n"
-"  level must be an integer value.  If not specified, defaults to 0."
-"  Always returns 0.\n";
+"  level must be an integer value.  If not specified, defaults to 0.\n";
 
 STANDARD_LOCAL_USER;
 
@@ -41,6 +56,9 @@ static int verbose_exec(struct ast_channel *chan, void *data)
 {
 	char *vtext;
 	int vsize;
+	struct localuser *u;
+
+	LOCAL_USER_ADD(u);
 
 	if (data) {
 		vtext = ast_strdupa((char *)data);
@@ -78,13 +96,20 @@ static int verbose_exec(struct ast_channel *chan, void *data)
 		}
 	}
 
+	LOCAL_USER_REMOVE(u);
+
 	return 0;
 }
 
 int unload_module(void)
 {
+	int res;
+
+	res = ast_unregister_application(app_verbose);
+
 	STANDARD_HANGUP_LOCALUSERS;
-	return ast_unregister_application(app_verbose);
+
+	return res;	
 }
 
 int load_module(void)
