@@ -1,7 +1,7 @@
 /*
- * Asterisk -- A telephony toolkit for Linux.
+ * Asterisk -- An open source telephony toolkit.
  *
- * PostgreSQL CDR logger 
+ * Copyright (C) 2003 - 2005
  *
  * Matthew D. Hardeman <mhardemn@papersoft.com> 
  * Adapted from the MySQL CDR logger originally by James Sharp 
@@ -9,20 +9,30 @@
  * Modified September 2003
  * Matthew D. Hardeman <mhardemn@papersoft.com>
  *
- * This program is free software, distributed under the terms of
- * the GNU General Public License.
+ * See http://www.asterisk.org for more information about
+ * the Asterisk project. Please do not directly contact
+ * any of the maintainers of this project for assistance;
+ * the project provides a web site, mailing lists and IRC
+ * channels for your use.
  *
+ * This program is free software, distributed under the terms of
+ * the GNU General Public License Version 2. See the LICENSE file
+ * at the top of the source tree.
+ */
+
+/*! \file
+ *
+ * \brief PostgreSQL CDR logger 
+ * 
+ * \author Matthew D. Hardeman <mhardemn@papersoft.com> 
+ *
+ * See also
+ * \arg \ref Config_cdr
+ * \arg http://www.postgresql.org/
+ * \ingroup cdr_drivers
  */
 
 #include <sys/types.h>
-#include <asterisk/config.h>
-#include <asterisk/options.h>
-#include <asterisk/channel.h>
-#include <asterisk/cdr.h>
-#include <asterisk/module.h>
-#include <asterisk/logger.h>
-#include "../asterisk.h"
-
 #include <stdio.h>
 #include <string.h>
 
@@ -32,13 +42,24 @@
 
 #include <libpq-fe.h>
 
+#include "asterisk.h"
+
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 7221 $")
+
+#include "asterisk/config.h"
+#include "asterisk/options.h"
+#include "asterisk/channel.h"
+#include "asterisk/cdr.h"
+#include "asterisk/module.h"
+#include "asterisk/logger.h"
+#include "asterisk.h"
+
 #define DATE_FORMAT "%Y-%m-%d %T"
 
 static char *desc = "PostgreSQL CDR Backend";
 static char *name = "pgsql";
 static char *config = "cdr_pgsql.conf";
-static char *pghostname = NULL, *pgdbname = NULL, *pgdbuser = NULL, *pgpassword = NULL, *pgdbsock = NULL, *pgdbport = NULL;
-static int hostname_alloc = 0, dbname_alloc = 0, dbuser_alloc = 0, password_alloc = 0, dbsock_alloc = 0, dbport_alloc = 0;
+static char *pghostname = NULL, *pgdbname = NULL, *pgdbuser = NULL, *pgpassword = NULL, *pgdbsock = NULL, *pgdbport = NULL, *table = NULL;
 static int connected = 0;
 
 AST_MUTEX_DEFINE_STATIC(pgsql_lock);
@@ -99,9 +120,14 @@ static int pgsql_log(struct ast_cdr *cdr)
 
 		ast_log(LOG_DEBUG,"cdr_pgsql: inserting a CDR record.\n");
 
-		snprintf(sqlcmd,sizeof(sqlcmd),"INSERT INTO cdr (calldate,clid,src,dst,dcontext,channel,dstchannel,lastapp,lastdata,duration,billsec,disposition,amaflags,accountcode,uniqueid,userfield) VALUES ('%s','%s','%s','%s','%s', '%s','%s','%s','%s',%i,%i,'%s',%i,'%s','%s','%s')",timestr,clid,cdr->src, cdr->dst, dcontext,channel, dstchannel, lastapp, lastdata,cdr->duration,cdr->billsec,ast_cdr_disp2str(cdr->disposition),cdr->amaflags, cdr->accountcode, uniqueid, userfield);
+		snprintf(sqlcmd,sizeof(sqlcmd),"INSERT INTO %s (calldate,clid,src,dst,dcontext,channel,dstchannel,"
+				 "lastapp,lastdata,duration,billsec,disposition,amaflags,accountcode,uniqueid,userfield) VALUES"
+				 " ('%s','%s','%s','%s','%s', '%s','%s','%s','%s',%d,%d,'%s',%d,'%s','%s','%s')",
+				 table,timestr,clid,cdr->src, cdr->dst, dcontext,channel, dstchannel, lastapp, lastdata,
+				 cdr->duration,cdr->billsec,ast_cdr_disp2str(cdr->disposition),cdr->amaflags, cdr->accountcode, uniqueid, userfield);
+		
 		ast_log(LOG_DEBUG,"cdr_pgsql: SQL command executed:  %s\n",sqlcmd);
-	
+		
 		/* Test to be sure we're still connected... */
 		/* If we're connected, and connection is working, good. */
 		/* Otherwise, attempt reconnect.  If it fails... sorry... */
@@ -157,38 +183,20 @@ static int my_unload_module(void)
 { 
 	if (conn)
 		PQfinish(conn);
-	conn = NULL;
-	connected = 0;
-	if (pghostname && hostname_alloc) {
+	if (pghostname)
 		free(pghostname);
-		pghostname = NULL;
-		hostname_alloc = 0;
-	}
-	if (pgdbname && dbname_alloc) {
+	if (pgdbname)
 		free(pgdbname);
-		pgdbname = NULL;
-		dbname_alloc = 0;
-	}
-	if (pgdbuser && dbuser_alloc) {
+	if (pgdbuser)
 		free(pgdbuser);
-		pgdbuser = NULL;
-		dbuser_alloc = 0;
-	}
-	if (pgdbsock && dbsock_alloc) {
+	if (pgdbsock)
 		free(pgdbsock);
-		pgdbsock = NULL;
-		dbsock_alloc = 0;
-	}
-	if (pgpassword && password_alloc) {
+	if (pgpassword)
 		free(pgpassword);
-		pgpassword = NULL;
-		password_alloc = 0;
-	}
-	if (pgdbport && dbport_alloc) {
+	if (pgdbport)
 		free(pgdbport);
-		pgdbport = NULL;
-		dbport_alloc = 0;
-	}
+	if (table)
+		free(table);
 	ast_cdr_unregister(name);
 	return 0;
 }
@@ -207,83 +215,69 @@ static int process_my_load_module(struct ast_config *cfg)
 	}
 
 	tmp = ast_variable_retrieve(cfg,"global","hostname");
-	if (tmp) {
-		pghostname = malloc(strlen(tmp) + 1);
-		if (pghostname != NULL) {
-			memset(pghostname, 0, strlen(tmp) + 1);
-			hostname_alloc = 1;
-			strncpy(pghostname, tmp, strlen(tmp));
-		} else {
-			ast_log(LOG_ERROR,"Out of memory error.\n");
-			return -1;
-		}
-	} else {
+	if (tmp == NULL) {
 		ast_log(LOG_WARNING,"PostgreSQL server hostname not specified.  Assuming localhost\n");
-		pghostname = "localhost";
+		tmp = "localhost";
+	}
+	pghostname = strdup(tmp);
+	if (pghostname == NULL) {
+		ast_log(LOG_ERROR,"Out of memory error.\n");
+		return -1;
 	}
 
 	tmp = ast_variable_retrieve(cfg,"global","dbname");
-	if (tmp) {
-		pgdbname = malloc(strlen(tmp) + 1);
-		if (pgdbname != NULL) {
-			memset(pgdbname, 0, strlen(tmp) + 1);
-			dbname_alloc = 1;
-			strncpy(pgdbname, tmp, strlen(tmp));
-		} else {
-			ast_log(LOG_ERROR,"Out of memory error.\n");
-			return -1;
-		}
-	} else {
+	if (tmp == NULL) {
 		ast_log(LOG_WARNING,"PostgreSQL database not specified.  Assuming asterisk\n");
-		pgdbname = "asteriskcdrdb";
+		tmp = "asteriskcdrdb";
+	}
+	pgdbname = strdup(tmp);
+	if (pgdbname == NULL) {
+		ast_log(LOG_ERROR,"Out of memory error.\n");
+		return -1;
 	}
 
 	tmp = ast_variable_retrieve(cfg,"global","user");
-	if (tmp) {
-		pgdbuser = malloc(strlen(tmp) + 1);
-		if (pgdbuser != NULL) {
-			memset(pgdbuser, 0, strlen(tmp) + 1);
-			dbuser_alloc = 1;
-			strncpy(pgdbuser, tmp, strlen(tmp));
-		} else {
-			ast_log(LOG_ERROR,"Out of memory error.\n");
-			return -1;
-		}
-	} else {
+	if (tmp == NULL) {
 		ast_log(LOG_WARNING,"PostgreSQL database user not specified.  Assuming root\n");
-		pgdbuser = "root";
+		tmp = "root";
+	}
+	pgdbuser = strdup(tmp);
+	if (pgdbuser == NULL) {
+		ast_log(LOG_ERROR,"Out of memory error.\n");
+		return -1;
 	}
 
 	tmp = ast_variable_retrieve(cfg,"global","password");
-	if (tmp) {
-		pgpassword = malloc(strlen(tmp) + 1);
-		if (pgpassword != NULL) {
-			memset(pgpassword, 0, strlen(tmp) + 1);
-			password_alloc = 1;
-			strncpy(pgpassword, tmp, strlen(tmp));
-		} else {
-			ast_log(LOG_ERROR,"Out of memory error.\n");
-			return -1;
-		}
-	} else {
+	if (tmp == NULL) {
 		ast_log(LOG_WARNING,"PostgreSQL database password not specified.  Assuming blank\n");
-		pgpassword = "";
+		tmp = "";
+	}
+	pgpassword = strdup(tmp);
+	if (pgpassword == NULL) {
+		ast_log(LOG_ERROR,"Out of memory error.\n");
+		return -1;
 	}
 
 	tmp = ast_variable_retrieve(cfg,"global","port");
-	if (tmp) {
-		pgdbport = malloc(strlen(tmp) + 1);
-		if (pgdbport != NULL) {
-			memset(pgdbport, 0, strlen(tmp) + 1);
-			dbport_alloc = 1;
-			strncpy(pgdbport, tmp, strlen(tmp));
-		} else {
-			ast_log(LOG_ERROR,"Out of memory error.\n");
-			return -1;
-		}
-	} else {
+	if (tmp == NULL) {
 		ast_log(LOG_WARNING,"PostgreSQL database port not specified.  Using default 5432.\n");
-		pgdbport = "5432";
+		tmp = "5432";
+	}
+	pgdbport = strdup(tmp);
+	if (pgdbport == NULL) {
+		ast_log(LOG_ERROR,"Out of memory error.\n");
+		return -1;
+	}
+
+	tmp = ast_variable_retrieve(cfg,"global","table");
+	if (tmp == NULL) {
+		ast_log(LOG_WARNING,"CDR table not specified.  Assuming cdr\n");
+		tmp = "cdr";
+	}
+	table = strdup(tmp);
+	if (table == NULL) {
+		ast_log(LOG_ERROR,"Out of memory error.\n");
+		return -1;
 	}
 
 	ast_log(LOG_DEBUG,"cdr_pgsql: got hostname of %s\n",pghostname);
@@ -293,7 +287,8 @@ static int process_my_load_module(struct ast_config *cfg)
 	ast_log(LOG_DEBUG,"cdr_pgsql: got user of %s\n",pgdbuser);
 	ast_log(LOG_DEBUG,"cdr_pgsql: got dbname of %s\n",pgdbname);
 	ast_log(LOG_DEBUG,"cdr_pgsql: got password of %s\n",pgpassword);
-
+	ast_log(LOG_DEBUG,"cdr_pgsql: got sql table name of %s\n",table);
+	
 	conn = PQsetdbLogin(pghostname, pgdbport, NULL, NULL, pgdbname, pgdbuser, pgpassword);
 	if (PQstatus(conn) != CONNECTION_BAD) {
 		ast_log(LOG_DEBUG,"Successfully connected to PostgreSQL database.\n");
@@ -316,13 +311,13 @@ static int my_load_module(void)
 {
 	struct ast_config *cfg;
 	int res;
-	cfg = ast_load(config);
+	cfg = ast_config_load(config);
 	if (!cfg) {
 		ast_log(LOG_WARNING, "Unable to load config for PostgreSQL CDR's: %s\n", config);
 		return 0;
 	}
 	res = process_my_load_module(cfg);
-	ast_destroy(cfg);
+	ast_config_destroy(cfg);
 	return res;
 }
 
@@ -344,7 +339,13 @@ int reload(void)
 
 int usecount(void)
 {
-	return connected;
+	/* To be able to unload the module */
+	if ( ast_mutex_trylock(&pgsql_lock) ) {
+		return 1;
+	} else {
+		ast_mutex_unlock(&pgsql_lock);
+		return 0;
+	}
 }
 
 char *key()
